/**
 * Copyright 2010 Västra Götalandsregionen
 *
 *   This library is free software; you can redistribute it and/or modify
 *   it under the terms of version 2.1 of the GNU Lesser General Public
 *   License as published by the Free Software Foundation.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the
 *   Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 *   Boston, MA 02111-1307  USA
 *
 */

package se.vgregion.portal.tasklist.services;

import java.lang.reflect.Field;

import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.config.BeanPostProcessor;
import org.springframework.util.ReflectionUtils;
import org.springframework.util.ReflectionUtils.FieldCallback;

/**
 * Auto injects the underlying implementation of logger into the bean with field having annotation Logger.
 * 
 */
public class LoggerInjector implements BeanPostProcessor {

    /**
     * 
     * {@inheritDoc}
     */
    public Object postProcessAfterInitialization(Object bean, String beanName) {
        return bean;
    }

    /**
     * 
     * {@inheritDoc}
     */
    public Object postProcessBeforeInitialization(final Object bean, String beanName) {
        ReflectionUtils.doWithFields(bean.getClass(), new FieldCallbackImpl(bean));
        return bean;
    }

    static class FieldCallbackImpl implements FieldCallback {
        private Object bean;

        FieldCallbackImpl(Object bean) {
            this.bean = bean;
        }

        public void doWith(Field field) throws IllegalAccessException {
            // make the field accessible if defined private
            ReflectionUtils.makeAccessible(field);
            if (field.getAnnotation(Logger.class) != null) {
                org.slf4j.Logger logger = LoggerFactory.getLogger(bean.getClass());
                field.set(bean, logger);
            }
        }
    }
}
